mod device;

#[macro_use]
extern crate glium;
extern crate nalgebra_glm as glm;

#[derive(Copy, Clone)]
struct Vertex {
    position: [f32; 3],
    color: [f32; 3],
}

implement_vertex!(Vertex, position, color);

fn main() {
    use glium::{glutin, Surface};
    
    let events_loop = glutin::event_loop::EventLoop::new();
    let wb = glutin::window::WindowBuilder::new()
        .with_title("Couch Game Engine");
    let cb = glutin::ContextBuilder::new();

    let display = glium::Display::new(wb, cb, &events_loop).unwrap();

    let vertex1 = Vertex { position: [-0.5, -0.5 , 0.0], color: [1.0, 0.0, 0.0] };
    let vertex2 = Vertex { position: [ 0.0,  0.5 , 0.0], color: [0.0, 1.0, 0.0] };
    let vertex3 = Vertex { position: [ 0.5, -0.25, 0.0], color: [0.0, 0.0, 1.0] };
    let shape = vec![vertex1, vertex2, vertex3];

    let vertex_buffer = glium::VertexBuffer::new(&display, &shape).unwrap();
    let indices = glium::index::NoIndices(glium::index::PrimitiveType::TrianglesList);

    let vertex_shader_src = include_str!("flat.vert");
    let fragment_shader_src = include_str!("flat.frag");
    let program = glium::Program::from_source(&display, vertex_shader_src, fragment_shader_src, None).unwrap();

    let mut device_manager = crate::device::DeviceManager::new();

    let mut offset = 0.0;

    events_loop.run(move |ev, _, control_flow| {
	if device_manager.is_pressed(&glutin::event::VirtualKeyCode::W) {
	    offset += 0.001;
	}
        let mvp = glm::perspective::<f32>(1.0, 45.0_f32.to_radians(), 0.2, 100.0);
        let mvp = glm::translate::<f32>(&mvp, &glm::vec3(0.0, 0.0, -10.0 + offset));
        let uniforms = uniform! {
            MVP: *mvp.as_ref(),
        };

        let mut target = display.draw();
        target.clear_color(0.0, 0.0, 0.0, 1.0);
        target.draw(&vertex_buffer, &indices, &program, &uniforms,
                    &Default::default()).unwrap();
        target.finish().unwrap();
        
        
        let next_frame_time = std::time::Instant::now() +
            std::time::Duration::from_nanos(16_666_667);
        *control_flow = glutin::event_loop::ControlFlow::WaitUntil(next_frame_time);
	match ev {
	    glutin::event::Event::WindowEvent { event, .. } => {
		if event == glutin::event::WindowEvent::CloseRequested {
                    *control_flow = glutin::event_loop::ControlFlow::Exit;
		}
	    },
	    glutin::event::Event::DeviceEvent { event, .. } => {
		device_manager.handle(event);
	    },
	    _ => (),
        };
    });
}
